/* This file is part of the KDE project
   Copyright (C) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef __connection_h__
#define __connection_h__

#include <qobject.h>

class QTextCodec;
class QSocket;

namespace IRC
{
class Connection : public QObject
{
    Q_OBJECT
public:
    Connection( QObject *parent, const char *name = 0 );
    virtual ~Connection();

    void connectToHost( const QString &host, Q_UINT16 port );

    void setEncoding( QTextCodec *codec );
    const QTextCodec * const encoding() const;

    virtual void writeLine( const QString &line );

    virtual QString peerDescription() const;
    virtual QString address() const;

signals:
    void connected();
    void connectionClosed();
    void lineRead( const QString &input );
    void error( const QString &errorMessage ); // ### fixme, bad API, should be errorcode

private slots:
    void tryReadLine();
    void socketError( int errorCode );

private:
    QTextCodec *m_codec;
    QSocket *m_socket;
};
};

#endif

/* vim: et sw=4
 */
