/* This file is part of the KDE project
   Copyright (C) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef MESSAGEFILTER_H
#define MESSAGEFILTER_H

#include "message.h"

#include <qobject.h>
#include <qptrlist.h>
#include <qdict.h>

namespace IRC
{
    
class Window;

class MessageFilter : public QObject
{
    Q_OBJECT
public:
    enum Result { ContinueFiltering, AbortFiltering, DiscardMessage };

    MessageFilter( QObject *parent, const char *name = 0 );
    virtual ~MessageFilter();

    virtual Result filterMessage( Message &msg, Window *window ) = 0;
};

class MessageFilterGroup : public MessageFilter
{
    Q_OBJECT
public:
    MessageFilterGroup( QObject *parent, const char *name = 0 );
    virtual ~MessageFilterGroup();

    virtual MessageFilter::Result filterMessage( Message &msg, Window *window );

    bool addFilter( MessageFilter *filter );
    bool removeFilter( MessageFilter *filter );

private slots:
    void filterDestroyed( QObject *object );

private:
    typedef QPtrList<MessageFilter> MessageFilterList;
    typedef QPtrListIterator<MessageFilter> MessageFilterListIterator;

    MessageFilterList m_filters;
};

class NamedMessageFilter : public MessageFilter
{
    Q_OBJECT
    Q_PROPERTY( QString messageType READ messageType );
public:
    NamedMessageFilter( const QString &messageType, QObject *parent, const char *name = 0 );

    virtual QString messageType() const;

private:
    QString m_messageType;
};

class NamedMessageFilterGroup : public MessageFilter
{
    Q_OBJECT
public:
    NamedMessageFilterGroup( QObject *parent, const char *name = 0 );
    virtual ~NamedMessageFilterGroup();

    virtual MessageFilter::Result filterMessage( Message &msg, Window *window );

    bool addFilter( const QString &name, MessageFilter *filter );
    bool addFilter( MessageFilter *filter );
    bool removeFilter( const QString &name );
    bool removeFilter( MessageFilter *filter );

private slots:
    void filterDestroyed( QObject *object );

private:
    typedef QDict<MessageFilter> MessageFilterMap;
    typedef QDictIterator<MessageFilter> MessageFilterMapIterator;

    MessageFilterMap m_filters;
};

};

#endif // MESSAGEFILTER_H
/* vim: et sw=4 ts=4
 */
