/* This file is part of the KDE project
   Copyright (C) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
#ifndef WINDOW_H
#define WINDOW_H

#include <qobject.h>

#include "message.h"
#include "commands.h"

namespace IRC
{

class Frontend;
class Command;

class Window : public QObject
{
    Q_OBJECT
public:
    Window( const QString &channel, Frontend *parent, const char *name = 0 );
    virtual ~Window();

    Frontend &frontend() { return *m_frontend; }
    const Frontend &frontend() const { return frontend(); }

    QString channel() const { return m_channel; }
    ChannelType type() const { return m_type; }

    const PersonList &users() const { return m_users; }

    virtual QString title() const = 0;

    bool isPrivateChat() const;

public slots:
    virtual void handlePrivmsgMessage( const IRC::PrivmsgMessage &msg );
    virtual void handleNoticeMessage( const IRC::NoticeMessage &msg );
    virtual void handleTopicMessage( const IRC::TopicMessage &msg );
    virtual void handleJoinMessage( const IRC::JoinMessage &msg );
    virtual void handlePartMessage( const IRC::PartMessage &msg );
    virtual void handleQuitMessage( const IRC::QuitMessage &msg );
    virtual void handleNickMessage( const IRC::NickMessage &msg );

    virtual void handleNumericMessage( const IRC::Message &msg );

    virtual void printMessage( const QString &text ) = 0;
    virtual void setTitle( const QString &text ) = 0;

    void updateUsersListFromNamesReply( const IRC::NamesReplyHandler::ChannelUsersMap &users );

signals:
    void textEntered( Window *window, const QString &text );

    void sendMessage( const IRC::Message &msg, QObject *replyHandler );
    void sendOneWayMessage( const IRC::Message &msg );
    void executeCommand( IRC::Command *cmd );

    void windowDestroyed( Window *thisWindow );

    void userUpdated( const IRC::Person &oldUser, const IRC::Person &newUser );
    void userRemoved( const IRC::Person &user );

protected slots:
    void processInput( const QString &text );
    void processCommand( const QString &text );
    void processCommand( const QString &command, const QStringList &arguments );
    void processCommand( const Message &msg );

private slots:
    void updateUserList();

private:
    void updateUsers( const NamesReplyHandler::NickInfoList &users );
    void updateUser( const NamesReplyHandler::NickInfo &user );

    PersonList::Iterator addUser( const QString &nick );
    void removeUser( const QString &nick );

    QString m_channel;
    Frontend *m_frontend;
    PersonList m_users;
    ChannelType m_type;
};

inline bool Window::isPrivateChat() const
{ return channel()[ 0 ] != '#'; }

}

#endif // WINDOW_H

/* vim: et sw=4 ts=4
 */
